import sys
import numpy
import warnings
import epop_ephemeris_library.common as common # To pull the Major/Minor version info.

DATABASE_HOST = 'cassiope-ephemeris.phys.ucalgary.ca'
DATABASE_PORT = 3306
DATABASE_NAME = 'cassiope_ephemeris'
DATABASE_USER = 'ephemeris_user'
DATABASE_PASSWORD = 'nokxw2'

OVERRIDE_VERSION_CHECK = False
_Version_Verified      = False

def Query( Query_String ) -> numpy.array:
  '''
  Query the Swarm-E Ephemeris Database.
  Initializes on import.
  '''
  import mysql.connector
  
  # Connect to the database. Raises a ConnectionError on any major failure.
  try:
    connection = mysql.connector.connect( 
      user=DATABASE_USER,
      password=DATABASE_PASSWORD,
      host=DATABASE_HOST,
      port=DATABASE_PORT,
      database=DATABASE_NAME,
      use_pure=True)
  except mysql.connector.Error as err:
    raise ConnectionError("Error: Could not connect to cassiope_ephemeris database.") from err
    
  # On connecting to the database, check that the versions match. 
  # This returns immediately if the check is suppressed or has been checked.
  _check_version( connection )

  # Send the query, fetch all data. (Risk of memory explosion on huge queries)
  cursor = connection.cursor()
  cursor.execute(Query_String)
  data = cursor.fetchall()
  # Close and clean up the connector.
  cursor.close()
  connection.close()
  
  return data
    
def _check_version( connection ):
  '''
  Verify that the major version of the ephemeris library matches the major revision
  Provide information if there is a minor version mismatch.
  
  Can override this automatic check by setting epop_ephemeris_library.common.OVERRIDE_VERSION_CHECK to True.
  '''
  global _Version_Verified # Load the 'has-checked' global as a funciton-var.

  # If the user has suppressed the verion check, or the version has already been checked, then return
  if OVERRIDE_VERSION_CHECK == True or _Version_Verified == True:
    return

  # Query the database.
  # connect to database and fetch data
  statement = ('SELECT `version` FROM `cassiope_ephemeris`.`library_version`')
  
  cursor = connection.cursor()
  cursor.execute( statement )
  data = cursor.fetchall()
  cursor.close()

  db_major_version, db_minor_version = data[0][0].split('.')
  if   int( db_major_version ) > int( common.MAJOR_VERSION ):
    raise ImportError("This version of epop_ephemeris_library is a major revision behind of the database. Database: v{:s}, this Library: v{:s}. Update the library to version v{s}, or suppress this check by setting database.OVERRIDE_VERSION_CHECK to True.".format( str(data[0][0]), str(common.VERSION) ) )
  elif int( db_major_version ) < int( common.MAJOR_VERSION ):
    warnings.warn('This version of epop_ephemeris_library is a major revision ahead of the database. Database: v{:s}, this Library: v{:s}. Changes may have been made to the database structure. Suggest downgrade so they match.'.format( str(data[0][0]), str(common.VERSION) ) )
    
  elif int( db_minor_version ) > int( common.MINOR_VERSION ):
    warnings.warn('This version of epop_ephemeris_library is a minor revision behind of the database. Database: v{:s}, this Library: v{:s}. If errors occur, suggest updated so they match.'.format( str(data[0][0]), str(common.VERSION) ) )
  elif int( db_minor_version ) < int( common.MINOR_VERSION ):
    warnings.warn('This version of epop_ephemeris_library is a minor revision ahead of the database. Database: v{:s}, this Library: v{:s}. If errors occur, suggest downgrade so they match.'.format( str(data[0][0]), str(common.VERSION) ) )
  
  # And if the Major version matches, only need to warn the once, so can skip any futher checks.
  _Version_Verified = True
